﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Interfaces;
using Library;
using Microsoft.AspNet.SignalR.Client;

namespace Client
{
    public class ClientApplication
    {
        private IHubProxy<IServer, IClient> Proxy { get; set; }

        private HubConnection HubConnection { get; set; }

        private string Url { get; set; }
        
        public ClientApplication(string url)
        {
            Url = url;
            SetupClient();
        }

        public void Run()
        {
            while (true)
            {
                string message = Console.ReadLine();

                //Wenn ESC eingegeben wird, soll abgebrochen werden.
                if (message != null && message.ToUpper() == "ESC")
                {
                    break;
                }

                try
                {
                    //Aufruf der ServerMethode NotifyOthersWithInformation
                    Proxy.CallAsync(hub => hub.NotityOthersWithInformation(message)).Wait();
                }
                catch (Exception)
                {
                    Console.WriteLine("ERROR: Could not send message.");
                }
            }
        }

        protected void SetupClient()
        {
            //erstellen der Connection auf der Default URL von SignalR
            HubConnection = new HubConnection(Url, useDefaultUrl: true); 

            //Verwenden des TypedHubProxy. So kann ich Interface Methoden verwenden.
            Proxy = HubConnection.CreateHubProxy<IServer, IClient>("MyHub"); 

            //Anmelden, wenn über NewMessage oder GetInformation Nachrichten vom Server geschickt werden.
            Proxy.SubscribeOn(hub => hub.GetInformation, (Information info) => Console.WriteLine(info));

            //Connection Starten.
            HubConnection.Start().ContinueWith(task =>
            {
                if (task.IsFaulted)
                {
                    if (task.Exception != null)
                    {
                        Console.WriteLine("There was an error opening the connection:{0}", task.Exception.GetBaseException());
                    }
                }
                else
                {
                    Console.WriteLine("Connected");
                }
            }).Wait();
        }
    }
}